-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("tagQuestGivers", "AceEvent-3.0")

-----------------------
--- Library loading ---
-----------------------
local LTQG = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-TagQuestGivers")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local select = select
local format = string.format
local GetGossipAvailableQuests, GetGossipActiveQuests = GetGossipAvailableQuests, GetGossipActiveQuests
local GetNumActiveQuests, GetNumAvailableQuests = GetNumActiveQuests, GetNumAvailableQuests
local GetActiveTitle, GetActiveLevel = GetActiveTitle, GetActiveLevel
local GetAvailableTitle, GetAvailableLevel = GetAvailableTitle, GetAvailableLevel

-----------------
--- variables ---
-----------------
module.modName = LTQG["Quests Givers Tag"]

function module:OnEnable()
	self:RegisterEvent("GOSSIP_SHOW")
	self:RegisterEvent("QUEST_GREETING")
end

local function questIconFaded(uid)
	if not uid then return false end
	return not (core.quests[uid].party[core.playerName].complete == 1 or core.quests[uid].numObjectives == 0)
end

local function gossipLoop(buttonindex, fade, ...)
	local numQuests = select('#', ...)
	for i=2, numQuests, 3 do
		local button = _G["GossipTitleButton"..buttonindex]
		local level = select(i, ...)
		button:SetText(format("[%s] %s", level == -1 and '*' or level, button:GetText()))
		if fade and questIconFaded(core.questsTitleToUid[select(i-1, ...)]) then
			_G["GossipTitleButton"..buttonindex.."GossipIcon"]:SetVertexColor(0.5, 0.5, 0.5, 0.5)
		else
			_G["GossipTitleButton"..buttonindex.."GossipIcon"]:SetVertexColor(1, 1, 1, 1)
		end
		buttonindex = buttonindex + 1
	end
	return buttonindex + 1
end

function module:GOSSIP_SHOW()
	local buttonIndex = 1
	if GetGossipAvailableQuests() then
		buttonIndex = gossipLoop(buttonIndex, false, GetGossipAvailableQuests())
	end
	if GetGossipActiveQuests() then
		buttonIndex = gossipLoop(buttonIndex, true, GetGossipActiveQuests())
	end
end

function module:QUEST_GREETING()
	local numActive, numAvailable = GetNumActiveQuests(), GetNumAvailableQuests()
	local title, level
	local o, GetTitle, GetLevel = 0, GetActiveTitle, GetActiveLevel
	for i=1, numActive + numAvailable do
		if i == numActive + 1 then
			o, GetTitle, GetLevel = numActive, GetAvailableTitle, GetAvailableLevel
		end
		title, level = GetTitle(i-o), GetLevel(i-o)
		_G["QuestTitleButton"..i]:SetText(format('[%s] %s', level == -1 and '*' or level, title))
		if questIconFaded(core.questsTitleToUid[title]) then
			_G["QuestTitleButton"..i.."QuestIcon"]:SetVertexColor(0.5, 0.5, 0.5, 0.5)
		else
			_G["QuestTitleButton"..i.."QuestIcon"]:SetVertexColor(1, 1, 1, 1)
		end
	end
end