-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("tooltips", "AceHook-3.0")

-----------------------
--- Library loading ---
-----------------------
local LT = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-Tooltip")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local pairs = pairs
local format, ceil = string.format, math.ceil

-----------------
--- Variables ---
-----------------
module.modName = LT["Tooltips"]
local prdb, options

-----------------------
--- Default profile ---
-----------------------
local defaults = {
	profile = {
		mobTooltip = true,
		mobItemDrop = true,
		itemTooltip = true,
	},
}

----------------------
--- Initialization ---
----------------------
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("tooltips", defaults, "Default")
	prdb = self.db.profile
end

---------------
--- Options ---
---------------
function module:getOptions()
	if not options then
		options = {
			type = "group",
			name = module.modName,
			desc = module.modName,
			get = function(info) 
				return prdb[info[#info]]
			end,
			set = function(info, value)
				prdb[info[#info]] = value
			end,
			args = {
				mobTooltip = {
					name = LT["Toggle tooltip modification from mobs"],
					type = "toggle",
					width = "full",
					order = 10,
				},
				mobItemDrop = {
					name = LT["Toggle quests item drop from mobs using MobMap data"],
					hidden = function()
						if MobMapDotParentFrame and prdb.mobTooltip then
							return false
						else
							return true
						end
					end,
					type = "toggle",
					width = "full",
					order = 11,
				},
				itemTooltip = {
					name = LT["Toggle tooltip modification from items"],
					type = "toggle",
					width = "full",
					order = 12,
				},
			},
		}
	end
	return options 
end

function module:OnEnable()
	self:HookScript(GameTooltip, "OnTooltipSetUnit")
	self:HookScript(GameTooltip, "OnTooltipSetItem")
end

function module:OnTooltipSetUnit(tooltip, ...)
	if not prdb.mobTooltip then return end
	local name = tooltip:GetUnit()
	if not name then return end
	if core.questsMobs[name] then
		for uid in pairs(core.questsMobs[name].quests) do
			tooltip:AddDoubleLine(format("|cffffffff%s|r", core.quests[uid].title), core.questsObjectives[uid][core.questsMobs[name].quests[uid]].qoStatus)
		end
	end
 	if not MobMapDotParentFrame or not prdb.mobItemDrop or not core.questsMobsItems[name] then return end
	for item in pairs (core.questsMobsItems[name].items) do
		local nGot, nNeed = 0, 0
		tooltip:AddLine(format(LT["|cffffffffDrop %s at %s%% chance|r"], item, core.questsMobsItems[name].items[item].dropRate))
		for uid in pairs(core.questsMobsItems[name].items[item].quests) do
			for player in pairs(core.questsObjectives[uid][core.questsMobsItems[name].items[item].quests[uid]].party) do
				nNeed = nNeed + core.questsObjectives[uid][core.questsMobsItems[name].items[item].quests[uid]].nNeed
				nGot = nGot + core.questsObjectives[uid][core.questsMobsItems[name].items[item].quests[uid]].party[player].nGot
			end
			tooltip:AddDoubleLine(format("  |cffffffff%s|r", core.quests[uid].title), core.questsObjectives[uid][core.questsMobsItems[name].items[item].quests[uid]].qoStatus)
		end
		tooltip:AddDoubleLine(format(LT["|cffffffffKills needed at %s%% drop chance|r"], core.questsMobsItems[name].items[item].dropRate), format("|cffffffff%s|r",ceil((nNeed - nGot)*100/core.questsMobsItems[name].items[item].dropRate)))
	end
end

function module:OnTooltipSetItem(tooltip, ...)
	if not prdb.itemTooltip then return end
	local name = tooltip:GetItem()
	if not name or not core.questsItems[name] then return end
	for uid in pairs(core.questsItems[name].quests) do
		tooltip:AddDoubleLine(format("|cffffffff%s|r", core.quests[uid].title), core.questsObjectives[uid][core.questsItems[name].quests[uid]].qoStatus)
	end
end